@@ -1,5 +1,4 @@
 import logging
-import unittest
 from unittest import mock
 
 from engineio import json
@@ -15,8 +14,8 @@ from socketio import server
 
 @mock.patch('socketio.server.engineio.Server', **{
     'return_value.generate_id.side_effect': [str(i) for i in range(1, 10)]})
-class TestServer(unittest.TestCase):
-    def tearDown(self):
+class TestServer:
+    def teardown_method(self):
         # restore JSON encoder, in case a test changed it
         packet.Packet.json = json
 
@@ -40,7 +39,7 @@ class TestServer(unittest.TestCase):
         def foo():
             pass
 
-        def bar():
+        def bar(reason):
             pass
 
         s.on('disconnect', bar)
@@ -463,6 +462,20 @@ class TestServer(unittest.TestCase):
         s.eio.send.assert_called_once_with('123', '4{"message":"fail_reason"}')
         assert s.environ == {'123': 'environ'}
 
+    def test_handle_connect_rejected_with_python_exception(self, eio):
+        s = server.Server()
+        handler = mock.MagicMock(
+            side_effect=ConnectionRefusedError()
+        )
+        s.on('connect', handler)
+        s._handle_eio_connect('123', 'environ')
+        s._handle_eio_message('123', '0')
+        assert not s.manager.is_connected('1', '/')
+        handler.assert_called_once_with('1', 'environ')
+        s.eio.send.assert_called_once_with(
+            '123', '4{"message":"Connection refused by server"}')
+        assert s.environ == {'123': 'environ'}
+
     def test_handle_connect_rejected_with_empty_exception(self, eio):
         s = server.Server()
         handler = mock.MagicMock(
@@ -511,8 +524,21 @@ class TestServer(unittest.TestCase):
         s.on('disconnect', handler)
         s._handle_eio_connect('123', 'environ')
         s._handle_eio_message('123', '0')
-        s._handle_eio_disconnect('123')
-        handler.assert_called_once_with('1')
+        s._handle_eio_disconnect('123', 'foo')
+        handler.assert_called_once_with('1', 'foo')
+        s.manager.disconnect.assert_called_once_with('1', '/',
+                                                     ignore_queue=True)
+        assert s.environ == {}
+
+    def test_handle_legacy_disconnect(self, eio):
+        s = server.Server()
+        s.manager.disconnect = mock.MagicMock()
+        handler = mock.MagicMock(side_effect=[TypeError, None])
+        s.on('disconnect', handler)
+        s._handle_eio_connect('123', 'environ')
+        s._handle_eio_message('123', '0')
+        s._handle_eio_disconnect('123', 'foo')
+        handler.assert_called_with('1')
         s.manager.disconnect.assert_called_once_with('1', '/',
                                                      ignore_queue=True)
         assert s.environ == {}
@@ -525,9 +551,9 @@ class TestServer(unittest.TestCase):
         s.on('disconnect', handler_namespace, namespace='/foo')
         s._handle_eio_connect('123', 'environ')
         s._handle_eio_message('123', '0/foo,')
-        s._handle_eio_disconnect('123')
+        s._handle_eio_disconnect('123', 'foo')
         handler.assert_not_called()
-        handler_namespace.assert_called_once_with('1')
+        handler_namespace.assert_called_once_with('1', 'foo')
         assert s.environ == {}
 
     def test_handle_disconnect_only_namespace(self, eio):
@@ -540,13 +566,14 @@ class TestServer(unittest.TestCase):
         s._handle_eio_message('123', '0/foo,')
         s._handle_eio_message('123', '1/foo,')
         assert handler.call_count == 0
-        handler_namespace.assert_called_once_with('1')
+        handler_namespace.assert_called_once_with(
+            '1', s.reason.CLIENT_DISCONNECT)
         assert s.environ == {'123': 'environ'}
 
     def test_handle_disconnect_unknown_client(self, eio):
         mgr = mock.MagicMock()
         s = server.Server(client_manager=mgr)
-        s._handle_eio_disconnect('123')
+        s._handle_eio_disconnect('123', 'foo')
 
     def test_handle_event(self, eio):
         s = server.Server(async_handlers=False)
@@ -597,7 +624,8 @@ class TestServer(unittest.TestCase):
         s._handle_eio_message('123', '2/bar,["msg","a","b"]')
         s._handle_eio_message('123', '2/foo,["my message","a","b","c"]')
         s._handle_eio_message('123', '2/bar,["my message","a","b","c"]')
-        s._trigger_event('disconnect', '/bar', sid_bar)
+        s._trigger_event('disconnect', '/bar', sid_bar,
+                         s.reason.CLIENT_DISCONNECT)
         connect_star_handler.assert_called_once_with('/bar', sid_bar)
         msg_foo_handler.assert_called_once_with(sid_foo, 'a', 'b')
         msg_star_handler.assert_called_once_with('/bar', sid_bar, 'a', 'b')
@@ -826,8 +854,8 @@ class TestServer(unittest.TestCase):
             def on_connect(self, sid, environ):
                 result['result'] = (sid, environ)
 
-            def on_disconnect(self, sid):
-                result['result'] = ('disconnect', sid)
+            def on_disconnect(self, sid, reason):
+                result['result'] = ('disconnect', sid, reason)
 
             def on_foo(self, sid, data):
                 result['result'] = (sid, data)
@@ -850,7 +878,8 @@ class TestServer(unittest.TestCase):
         s._handle_eio_message('123', '2/foo,["baz","a","b"]')
         assert result['result'] == ('a', 'b')
         s.disconnect('1', '/foo')
-        assert result['result'] == ('disconnect', '1')
+        assert result['result'] == ('disconnect', '1',
+                                    s.reason.SERVER_DISCONNECT)
 
     def test_catchall_namespace_handler(self, eio):
         result = {}
@@ -886,7 +915,7 @@ class TestServer(unittest.TestCase):
         assert result['result'] == ('disconnect', '1', '/foo')
 
     def test_bad_namespace_handler(self, eio):
-        class Dummy(object):
+        class Dummy:
             pass
 
         class AsyncNS(namespace.Namespace):
@@ -948,7 +977,7 @@ class TestServer(unittest.TestCase):
         # Warning: this test cannot run in parallel with other tests, as it
         # changes the JSON encoding/decoding functions
 
-        class CustomJSON(object):
+        class CustomJSON:
             @staticmethod
             def dumps(*args, **kwargs):
                 return '*** encoded ***'
