@@ -1,11 +1,10 @@
-import unittest
 from unittest import mock
 import pytest
 from socketio import SimpleClient
 from socketio.exceptions import SocketIOError, TimeoutError, DisconnectedError
 
 
-class TestSimpleClient(unittest.TestCase):
+class TestSimpleClient:
     def test_constructor(self):
         client = SimpleClient(1, '2', a='3', b=4)
         assert client.client_args == (1, '2')
@@ -15,10 +14,34 @@ class TestSimpleClient(unittest.TestCase):
         assert not client.connected
 
     def test_connect(self):
+        mock_client = mock.MagicMock()
+        original_client_class = SimpleClient.client_class
+        SimpleClient.client_class = mock_client
+
         client = SimpleClient(123, a='b')
-        with mock.patch('socketio.simple_client.Client') as mock_client:
+        client.connect('url', headers='h', auth='a', transports='t',
+                       namespace='n', socketio_path='s', wait_timeout='w')
+        mock_client.assert_called_once_with(123, a='b')
+        assert client.client == mock_client()
+        mock_client().connect.assert_called_once_with(
+            'url', headers='h', auth='a', transports='t',
+            namespaces=['n'], socketio_path='s', wait_timeout='w')
+        mock_client().event.call_count == 3
+        mock_client().on.assert_called_once_with('*', namespace='n')
+        assert client.namespace == 'n'
+        assert not client.input_event.is_set()
+
+        SimpleClient.client_class = original_client_class
+
+    def test_connect_context_manager(self):
+        mock_client = mock.MagicMock()
+        original_client_class = SimpleClient.client_class
+        SimpleClient.client_class = mock_client
+
+        with SimpleClient(123, a='b') as client:
             client.connect('url', headers='h', auth='a', transports='t',
-                           namespace='n', socketio_path='s', wait_timeout='w')
+                           namespace='n', socketio_path='s',
+                           wait_timeout='w')
             mock_client.assert_called_once_with(123, a='b')
             assert client.client == mock_client()
             mock_client().connect.assert_called_once_with(
@@ -29,21 +52,7 @@ class TestSimpleClient(unittest.TestCase):
             assert client.namespace == 'n'
             assert not client.input_event.is_set()
 
-    def test_connect_context_manager(self):
-        with SimpleClient(123, a='b') as client:
-            with mock.patch('socketio.simple_client.Client') as mock_client:
-                client.connect('url', headers='h', auth='a', transports='t',
-                               namespace='n', socketio_path='s',
-                               wait_timeout='w')
-                mock_client.assert_called_once_with(123, a='b')
-                assert client.client == mock_client()
-                mock_client().connect.assert_called_once_with(
-                    'url', headers='h', auth='a', transports='t',
-                    namespaces=['n'], socketio_path='s', wait_timeout='w')
-                mock_client().event.call_count == 3
-                mock_client().on.assert_called_once_with('*', namespace='n')
-                assert client.namespace == 'n'
-                assert not client.input_event.is_set()
+        SimpleClient.client_class = original_client_class
 
     def test_connect_twice(self):
         client = SimpleClient(123, a='b')
@@ -55,7 +64,7 @@ class TestSimpleClient(unittest.TestCase):
 
     def test_properties(self):
         client = SimpleClient()
-        client.client = mock.MagicMock(transport='websocket')
+        client.client = mock.MagicMock(transport=lambda: 'websocket')
         client.client.get_sid.return_value = 'sid'
         client.connected_event.set()
         client.connected = True
@@ -121,6 +130,16 @@ class TestSimpleClient(unittest.TestCase):
         client.client.call.assert_called_with('foo', 'bar', namespace='/',
                                               timeout=60)
 
+    def test_call_timeout(self):
+        client = SimpleClient()
+        client.connected_event.set()
+        client.connected = True
+        client.client = mock.MagicMock()
+        client.client.call.side_effect = TimeoutError()
+
+        with pytest.raises(TimeoutError):
+            client.call('foo', 'bar')
+
     def test_receive_with_input_buffer(self):
         client = SimpleClient()
         client.input_buffer = ['foo', 'bar']
